local assert, error, ipairs, next, pairs, select, tonumber, tostring, type, unpack, pcall, xpcall =
      assert, error, ipairs, next, pairs, select, tonumber, tostring, type, unpack, pcall, xpcall
local math, string, table = math, string, table
local find, format, gmatch, gsub, tolower, match, toupper, join, split, trim =
      string.find, string.format, string.gmatch, string.gsub, string.lower, string.match, string.upper, string.join, string.split, string.trim
local concat, insert, maxn, remove, sort = table.concat, table.insert, table.maxn, table.remove, table.sort
local max, min, abs, ceil, floor = math.max, math.min, math.abs, math.ceil, math.floor

local AceLibrary = AceLibrary
local MerchantFrame = MerchantFrame

local _G = getfenv()
do
  local function gerr(t, k) error(format("Reference to global %q", k), 2) end
  setfenv(1, setmetatable({}, {__index = gerr, __newindex = gerr}))
end

local L = AceLibrary("AceLocale-2.2"):new("ItemPriceTooltip")

L:RegisterTranslations("enUS", function() return {
    ["Sells for"] = true,
  }
end)

L:RegisterTranslations("deDE", function() return {
    ["Sells for"] = "Verkauf f\195\188r",
  }
end)

L:RegisterTranslations("frFR", function() return {
    ["Sells for"] = "Vendu pour",
  }
end)

L:RegisterTranslations("koKR", function() return {
    ["Sells for"] = "판매 가격",
  }
end)

L:RegisterTranslations("zhTW", function() return {
    ["Sells for"] = "賣出價",
  }
end)

local ItemPriceTooltip = AceLibrary("AceAddon-2.0"):new("AceHook-2.1", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0")

-- ItemPriceTooltip:SetDebugLevel(3)
-- ItemPriceTooltip:SetDebugging(false)

local ItemPrice = AceLibrary("ItemPrice-1.0")
local Abacus = AceLibrary("Abacus-2.0")
local Crayon = AceLibrary("Crayon-2.0")

ItemPriceTooltip:RegisterDB("ItemPriceTooltipDB")

local function dbg() return ItemPriceTooltip:IsDebugging() end
local function debug(...) ItemPriceTooltip:Debug(...) end
local function ldebug(level, ...) ItemPriceTooltip:LevelDebug(level, ...) end

local Hooks = {}
do
  local SellsForText = Crayon:Colorize("7fff7f", L["Sells for"]) .. " "
                    .. Crayon:Colorize("ffffaa", "(")
                    .. Crayon:Colorize("ffffff", "%d") .. " "
                    .. Crayon:Colorize("aaaaaa", "x") .. " %s"
                    .. Crayon:Colorize("ffffaa", ")")

  local function formatPriceText(price, count)
    local single = Abacus:FormatMoneyShort(price, true)
    local stack = Abacus:FormatMoneyFull(price * count, true)
    return format(SellsForText, count, single), stack
  end

  local function addPrice(tooltip, count)
    if dbg() then debug("addPrice(%s, %s)", tostring(tooltip), tostring(count)) end
    count = count or 1
    local _, item = tooltip:GetItem()
    if not item then return end
    local price = ItemPrice:GetPrice(item)
    if not price then return end
    local left, right = formatPriceText(price, count)
    tooltip:AddDoubleLine(left, right)
    if tooltip:IsShown() then tooltip:Show() end
  end

  function Hooks:SetAuctionItem(type, index)
    local _, _, count = _G.GetAuctionItemInfo(type, index)
    addPrice(self, count)
  end

  function Hooks:SetAuctionSellItem()
    local _, _, count = _G.GetAuctionSellItemInfo()
    addPrice(self, count)
  end

  function Hooks:SetBagItem(bag, slot)
    if MerchantFrame:IsShown() then return end
    local _, count = _G.GetContainerItemInfo(bag, slot)
    addPrice(self, count)
  end

  function Hooks:SetCraftItem(skill, slot)
    local count = 1
    if slot then
      count = select(3, _G.GetCraftReagentInfo(skill, slot))
    end
    addPrice(self, count)
  end

  function Hooks:SetHyperlink(link)
    addPrice(self, 1)
  end

  function Hooks:SetInboxItem(index)
    local _, _, count = _G.GetInboxItem(index)
    addPrice(self, count)
  end

  function Hooks:SetInventoryItem(unit, slot)
    if type(slot) ~= "number" or slot < 0 then return end
    local count = 1
    if slot < 20 or slot > 39 and slot < 68 then
      count = _G.GetInventoryItemCount(unit, slot)
    end
    addPrice(self, count)
  end

  function Hooks:SetLootItem(slot)
    local _, _, count = _G.GetLootSlotInfo(slot)
    addPrice(self, count)
  end

  function Hooks:SetLootRollItem(rollID)
    local _, _, count = _G.GetLootRollItemInfo(rollID)
    addPrice(self, count)
  end

  function Hooks:SetMerchantCostItem(index, item) -- FIXME: Needs testing!
    local _, count = _G.GetMerchantItemCostItem(index, item)
    addPrice(self, count)
  end

  function Hooks:SetMerchantItem(slot)
    local _, _, _, count = _G.GetMerchantItemInfo(slot)
    addPrice(self, count)
  end

  function Hooks:SetQuestItem(type, slot)
    local _, _, count = _G.GetQuestItemInfo(type, slot)
    addPrice(self, count)
  end

  function Hooks:SetQuestLogItem(type, index)
    local _, _, count = _G.GetQuestLogRewardInfo(index)
    addPrice(self, count)
  end

  function Hooks:SetSendMailItem()
    local _, _, count = _G.GetSendMailItem()
    addPrice(self, count)
  end

  function Hooks:SetSocketedItem() -- FIXME: Needs testing!
    addPrice(self, 1)
  end

  function Hooks:SetExistingSocketGem() -- FIXME: Needs testing!
    addPrice(self, 1)
  end

  function Hooks:SetSocketGem() -- FIXME: Needs testing!
    addPrice(self, 1)
  end

  function Hooks:SetTradePlayerItem()
    local _, _, count = _G.GetTradePlayerItemInfo()
    addPrice(self, count)
  end

  function Hooks:SetTradeSkillItem(skill, slot)
    local count = 1
    if slot then
      count = select(3, _G.GetTradeSkillReagentInfo(skill, slot))
    end
    addPrice(self, count)
  end

  function Hooks:SetTradeTargetItem()
    local _, _, count = _G.GetTradeTargetItemInfo()
    addPrice(self, count)
  end
end



function ItemPriceTooltip:OnInitialize()
  self:RegisterChatCommand({ "/itempricetooltip", "/ipt" })
end

function ItemPriceTooltip:OnEnable(first)
  self:installHooks(_G.GameTooltip, Hooks)
  self:installHooks(_G.ItemRefTooltip, Hooks)
end

function ItemPriceTooltip:installHooks(tooltip, hooks)
  for name, func in pairs(hooks) do
    if type(tooltip[name]) == "function" then
      self:SecureHook(tooltip, name, func)
    end
  end
end

